<?php
/**
 * Classes para controles em formularios
 *
 * @file nwx_form_ctrl.php
 * @author Gonalo Franco
 * @version $Id: nwx_form_ctrl.php 924 2007-01-11 18:36:37Z goncalo $
 */

/**
 * Classe representando um controle generico em um formulario
 */
abstract class nwx_form_ctrl {

  public $id;
  public $nome;
  public $label;
  public $valor;
  public $atributos;
  public $br;
  public $tipo;
  public $tam;
  public $dec;
  public $nulo;
  public $read_only;
  public $automatico;
  public $jscript;
  public $form;
  public $lov;
  public $upload = false;
  public $ajuda;
  public $bloco;
  public $mostra_label;
  public $link;
  
  public $ref_sql;
  public $ref_campo;
  public $ref_atrib;

  /**
   * Contrutor: Cria o objeto
   *
   * @param $id Identificacao do campo
   * @param $tipo Tipo de dados
   * @param $tam Tamanho
   * @param $dec Decimais
   * @param $nulo Se pode ser nulo
   * @param $ro Se read-only
   */
  function __construct($id, $tipo = 'C', $tam = 0, $dec = 0, $nulo = true, $ro = false) {

    $this->id = $id;
    $this->tipo = strtoupper($tipo);
    $this->tam = $tam;
    $this->dec = $dec;
    $this->nulo = $nulo;
    $this->read_only = $ro;
    $this->automatico = false;
    $this->jscript = '';
    $this->ref_sql = '';
    $this->ref_campo = '';
    $this->ref_atrib = '';
    $this->lov = '';
    $this->ajuda = '';
    $this->mostra_label = true;
    $this->link = '';
    
  }

  /**
   * Verifica validade do controle
   *
   * @param $valor Conteudo do campo
   */
  function verifica($valor) {

    // Verifica se e obrigatorio e foi preenchido
    if (!$this->nulo and $valor == '') {
      $dom = textdomain(null);
      textdomain('nwx');
      $txt = _('Obrigatrio');
      textdomain($dom);
      return $this->label . ' : ' . $txt;
    }

    $valor = trim($valor);

    switch ($this->tipo) {
      // Campo alfanumerico
      case 'C' :
        // Verifica tam. maximo
        if (strlen($valor) > $this->tam && !empty($this->tam)) {
          $dom = textdomain(null);
          textdomain('nwx');
          $txt1 = _('Mximo de');
          $txt2 = _('caracteres');
          textdomain($dom);
          return $this->label . ' : ' . $txt1 . ' '.$this->tam.' '.$txt2;
        }
        break;
      // Campo data : formato DDMMYYYY ou DD/MM/YYYY
      case 'D' :
        if (empty($valor))
          return '';
        if (!nwx_time::data_valida($valor)) {
          $dom = textdomain(null);
          textdomain('nwx');
          $txt1 = _('Data invlida');
          textdomain($dom);
          return $this->label . ' : ' . $txt1;
        }
        break;
      // Campo data/hora : formato DDMMYYYY ou DD/MM/YYYY + HHMM ou HH:MM
      case 'T' :
        if (empty($valor))
          return '';
        if (!nwx_time::data_valida($valor)) {
          $dom = textdomain(null);
          textdomain('nwx');
          $txt1 = _('Data invlida');
          textdomain($dom);
          return $this->label . ' : ' . $txt1;
        }
        break;
      // Campo hora : formato HH:MM
      case 'H' :
        if (!$valor)
          return '';
        if (strlen($valor) != 5) return $this->label . ' : ' . _('Hora invlida');
        $d = explode(":",$valor);
        if (sizeof($d) != 2) return $this->label . ' : ' . _('Hora invlida');
        if (($d[0] < '00' || $d[0] > '24') || $d[1] < '00' || $d[1] > '59') {
          $dom = textdomain(null);
          textdomain('nwx');
          $txt1 = _('Hora invlida');
          textdomain($dom);
          return $this->label . ' : ' . $txt1;
        }
        break;
      // Campo numerico : verifica tamanho e precisao
      case 'N' :
        if (empty($valor))
          return '';
        $valor = str_replace(',','.',$valor);  
        if (!is_numeric($valor)) {
          $dom = textdomain(null);
          textdomain('nwx');
          $txt1 = _('Numero invlido');
          textdomain($dom);
          return $this->label . ' : ' . $txt1;
        }
        if ($this->dec != '' and $this->dec == 0 and !is_int(1 + $valor)) {
          $dom = textdomain(null);
          textdomain('nwx');
          $txt1 = _('Numero no inteiro');
          textdomain($dom);
          return $this->label . ' : ' . $txt1;
        }
        $mi = $this->tam - $this->dec;
        $t = ($valor[0] == '-' or $valor[0] == '+') ? 1 : 0;
        $pos = strpos('x'.$valor,'.');
        if (is_int($pos)) {
          --$pos;
          $tpi = $pos - $t;
          $tpd = strlen($valor) - $pos - 1;
        } else {
          $tpi = strlen($valor) - $t;
          $tpd = 0;
        }
        if ($tpi > $mi) {
          $dom = textdomain(null);
          textdomain('nwx');
          $txt1 = _('Numero maior que')." $mi "._('posies');
          textdomain($dom);
          return $this->label . ' : ' . $txt1;
        }
        if ($this->dec == '') {
          if ($tpd > $mi - $tpi) {
            $dom = textdomain(null);
            textdomain('nwx');
            $txt1 = _('Preciso mxima de').' '.($mi - $tpi).' '._('decimais');
            textdomain($dom);
            return $this->label . ' : ' . $txt1;
          }
        } else {
          if ($tpd > $this->dec) {
            $dom = textdomain(null);
            textdomain('nwx');
            $txt1 = _('Preciso mxima de').' '.$this->dec.' '._('decimais');
            textdomain($dom);
            return $this->label . ' : ' . $txt1;
          }
        }
        break;
      case 'CPF' :
        // Verifica tam. maximo
        if (strlen($valor) > $this->tam) {
          $dom = textdomain(null);
          textdomain('nwx');
          $txt1 = _('Mximo de').' '.$this->tam.' '._('caracteres');
          textdomain($dom);
          return $this->label . ' : ' . $txt1;
        }
        if ($valor != '' && !VerifCPF($valor)) {
          $dom = textdomain(null);
          textdomain('nwx');
          $txt1 = _('CPF invlido');
          textdomain($dom);
          return $this->label . ' : ' . $txt1;
        }
        break;
      case 'CNPJ' :
        // Verifica tam. maximo
        if (strlen($valor) > $this->tam) {
          $dom = textdomain(null);
          textdomain('nwx');
          $txt1 = _('Mximo de').' '.$this->tam.' '._('caracteres');
          textdomain($dom);
          return $this->label . ' : ' . $txt1;
        }
        if ($valor != '' && !VerifCNPJ($valor)) {
          $dom = textdomain(null);
          textdomain('nwx');
          $txt1 = _('CNPJ invlido');
          textdomain($dom);
          return $this->label . ' : ' . $txt1; 
        }
        break;
      case 'CPF_CNPJ' :
        // Verifica tam. maximo
        if (strlen($valor) > $this->tam) {
          $dom = textdomain(null);
          textdomain('nwx');
          $txt1 = _('Mximo de').' '.$this->tam.' '._('caracteres');
          textdomain($dom);
          return $this->label . ' : ' . $txt1;
        }
        if ($valor != '' && !VerifCPF($valor) && !VerifCNPJ($valor)) {
          $dom = textdomain(null);
          textdomain('nwx');
          $txt1 = _('CPF ou CNPJ invlido');
          textdomain($dom);
          return $this->label . ' : ' . $txt1;
        }
        break;
      case 'EMAIL' :
        // Verifica tam. maximo
        if (strlen($valor) > $this->tam) {
          $dom = textdomain(null);
          textdomain('nwx');
          $txt1 = _('Mximo de').' '.$this->tam.' '._('caracteres');
          textdomain($dom);
          return $this->label . ' : ' . $txt1;
        }
        $patt = "^([-!#\$%&'*+./0-9=?A-Z^_`a-z{|}~])+@([-!#\$%&'*+/0-9=?A-Z^_`a-z{|}~]+\\.)+[a-zA-Z]{2,6}\$";
        if ($valor != '' && !eregi($patt,$valor)) {
          $dom = textdomain(null);
          textdomain('nwx');
          $txt1 = _('Formato invlido para e-mail');
          textdomain($dom);
          return $this->label . ' : ' . $txt1;
        }
        break;
    }
    return '';

  }

  /**
   * Campo de display
   *
   */
  function display($sql, $campo, $atrib) {
    
    $this->ref_sql = $sql;
    $this->ref_campo = $campo;
    $this->ref_atrib = $atrib;
    
  }

  /**
   * Mostra o controle no form
   *
   */
  function mostra($ro, $nv) {

    $this->monta($ro, $nv);

  }

  /**
   * Mostra o controle no form
   */
  function mostra_form($form, $atr = '') {

    $this->nome = $this->id;
    $this->form = $form;
    $this->atributos = $atr;
    $this->monta(false, false);
    echo $this->jscript;

  }

  /**
   * Mostra botao de ajuda do campo
   */
  function ajuda() {

    $css = $_SESSION['nwx_estilo'];
    $fig = "/nwx/estilos/$css/help.gif";
    if (!empty($this->ajuda)) {
      echo "<img class=\"ctlimg\" border=\"0\" src=\"$fig\" ".nwx_hctl($this->label,$this->ajuda)."></a>";
    }
    if ($this->link) {
      echo " <a href=\"{$this->link[0]}\" target=\"_blank\">[{$this->link[1]}]</a>";
    }
    
  }

  
  /**
   * Monta o conteudo do controle
   *
   */
  abstract function monta($ro, $nv);

} //class nw_form_ctrl
// vim:et:sw=2:ts=2:nowrap:
